/****************************************************************************
 *  Copyright (C) 2009-2010 by Martin Engelmann, murphi.oss@googlemail.com  *
 *                                                                          *
 *  This program is free software; you can redistribute it and/or           *
 *  modify it under the terms of the GNU General Public License as          *
 *  published by the Free Software Foundation; either version 2 of          *
 *  the License or (at your option) any later version.                      *
 *                                                                          *
 *  This program is distributed in the hope that it will be useful,         *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>    *
 ****************************************************************************/

#include "kookie.h"
#include "recipedisplaywidget.h"
#include "recipe.h"
#include "settings.h"
#include "recipereaderwriterrecipexml.h"

#include <QtGui/QDropEvent>
#include <QtGui/QPainter>

#include <kconfigdialog.h>
#include <kstatusbar.h>

#include <kaction.h>
#include <kactioncollection.h>
#include <kstandardaction.h>
#include <kfiledialog.h>
#include <kurl.h>
#include <kmessagebox.h>

#include <KDE/KLocale>

Kookie::Kookie()
        : KXmlGuiWindow(),
        m_view(new RecipeDisplayWidget(this)),
        m_printer(0)
{
    // accept dnd
    setAcceptDrops(true);

    // tell the KXmlGuiWindow that this is indeed the main widget
    setCentralWidget(m_view);

    // then, setup our actions
    setupActions();

    // add a status bar
    statusBar()->show();

    // a call to KXmlGuiWindow::setupGUI() populates the GUI
    // with actions, using KXMLGUI.
    // It also applies the saved mainwindow settings, if any, and ask the
    // mainwindow to automatically save settings if changed: window size,
    // toolbar position, icon size, etc.
    setupGUI();

    connect(m_view, SIGNAL(modified(bool)), this, SLOT(setWindowModified(bool)));
    connect(m_view, SIGNAL(modified(bool)), this, SLOT(slotUpdateWindowTitle()));

    slotFileNew();
}

Kookie::~Kookie()
{
}

void Kookie::closeEvent(QCloseEvent* event)
{
    if (mayBeSave(i18nc("@title:window", "Close recipe")) == KMessageBox::Cancel) {
        event->ignore();
    } else {
        event->accept();
    }
}

void Kookie::slotQuit()
{
    if (mayBeSave(i18nc("@title:window", "Close recipe")) == KMessageBox::Cancel) return;

    qApp->quit();
}

void Kookie::slotOptionsPreferences()
{
    // The preference dialog is derived from prefs_base.ui
    //
    // compare the names of the widgets in the .ui file
    // to the names of the variables in the .kcfg file
    //avoid to have 2 dialogs shown
    if (KConfigDialog::showDialog("settings"))  {
        return;
    }
    KConfigDialog *dialog = new KConfigDialog(this, "settings", Settings::self());
    QWidget *generalSettingsDlg = new QWidget;
    ui_prefs_base.setupUi(generalSettingsDlg);
    dialog->addPage(generalSettingsDlg, i18nc("@title:tab General settings", "General"), "package_setting");
//    connect(dialog, SIGNAL(settingsChanged(QString)), m_view, SLOT(settingsChanged()));
    dialog->setAttribute(Qt::WA_DeleteOnClose);
    dialog->show();
}

void Kookie::slotFileNew()
{
    if (mayBeSave(i18nc("@title:window", "New recipe")) == KMessageBox::Cancel) return;

    m_filename.clear();
    m_view->setRecipe(Recipe(i18nc("@title Title of newly created recipe", "Unnamed")));
}

void Kookie::slotOpenFile()
{
    if (mayBeSave(i18nc("@title:window", "Open recipe")) == KMessageBox::Cancel) return;

    KUrl openFilename(KFileDialog::getOpenUrl());
    if (openFilename.isEmpty()) return;

    Recipe recipe;
    RecipeReaderWriter* reader = new RecipeReaderWriterRecipeXml();
    bool success = reader->readRecipe(openFilename, recipe);
    delete reader;

    if (!success) return;

    m_filename = openFilename;
    m_view->setRecipe(recipe);
}

void Kookie::slotSaveFile()
{
    if (!m_filename.isEmpty()) {
        saveFileAs(m_filename);
    } else {
        slotSaveFileAs();
    }
}

void Kookie::slotSaveFileAs()
{
    saveFileAs(KFileDialog::getSaveUrl());
}

void Kookie::slotUpdateWindowTitle()
{
    setWindowTitle(i18nc("@title:window", "%1[*] - %2",
                         m_view->recipe()->title(),
                         KGlobal::mainComponent().componentName()));
}

void Kookie::saveFileAs(const KUrl& filename)
{
    if (filename.isEmpty()) return;

    m_filename = filename;

    RecipeReaderWriter* writer = new RecipeReaderWriterRecipeXml();
    writer->writeRecipe(*m_view->recipe(), filename);
    delete writer;

    setWindowModified(false);
}

void Kookie::setupActions()
{
    KStandardAction::quit(this, SLOT(slotQuit()), actionCollection());
//    KStandardAction::preferences(this, SLOT(slotOptionsPreferences()), actionCollection());

    KStandardAction::openNew(this, SLOT(slotFileNew()), actionCollection());
    KStandardAction::open(this, SLOT(slotOpenFile()), actionCollection());
    KStandardAction::save(this, SLOT(slotSaveFile()), actionCollection());
    KStandardAction::saveAs(this, SLOT(slotSaveFileAs()), actionCollection());

    KAction* editRecipe = new KAction(KIcon("document-edit"), i18nc("@action", "Edit recipe"), this);
    editRecipe->setShortcut(Qt::Key_F2);
    actionCollection()->addAction(QLatin1String("recipe-edit"), editRecipe);
    connect(editRecipe, SIGNAL(triggered(bool)), m_view, SLOT(slotEdit()));
}

int Kookie::mayBeSave(const QString& title)
{
    if (isWindowModified()) {
        int result = KMessageBox::warningYesNoCancel(this,
                     i18nc("@info", "The recipe <resource>%1</resource> has not been saved yet.<nl/>"
                           "Do you want to save or discard your changes?", m_view->recipe()->title()),
                     title, KStandardGuiItem::save(), KStandardGuiItem::discard());
        if (result == KMessageBox::Yes) {
            slotSaveFile();
        }
        return result;
    }
    return KMessageBox::Yes;
}

#include "kookie.moc"
