/****************************************************************************
 *  Copyright (C) 2009-2010 by Martin Engelmann, murphi.oss@googlemail.com  *
 *                                                                          *
 *  This program is free software; you can redistribute it and/or           *
 *  modify it under the terms of the GNU General Public License as          *
 *  published by the Free Software Foundation; either version 2 of          *
 *  the License or (at your option) any later version.                      *
 *                                                                          *
 *  This program is distributed in the hope that it will be useful,         *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>    *
 ****************************************************************************/

#include "recipe.h"

#include <QList>
#include <QStringList>

class RecipePrivate
{
public:
    QString             title;
    QString             author;
    QString             blurb;
    QString             effort;
    QString             genre;
    QString             preparationTime;
    QString             source;
    QString             yield;
    QList<Ingredient>   ingredients;
    QStringList         steps;

    bool isIngredientIndexValid(int index) const {
        return isIndexValid(index, ingredients);
    }

    bool isStepIndexValid(int index) const {
        return isIndexValid(index, steps);
    }

private:
    template<class T> bool isIndexValid(int index, const QList<T>& list) const {
        return index >= 0 && index < list.size();
    }
};


Recipe::Recipe()
        : d(new RecipePrivate())
{
}


Recipe::Recipe(const QString& title)
        : d(new RecipePrivate())
{
    setTitle(title);
}

Recipe::Recipe(const Recipe& rhs)
        : d(new RecipePrivate(*(rhs.d)))
{
}

Recipe::~Recipe()
{
    delete d;
}


Recipe& Recipe::operator=(const Recipe & rhs)
{
    if (this == &rhs) return *this;

    delete d;
    d = new RecipePrivate(*(rhs.d));
    return *this;
}


QString Recipe::title() const
{
    return d->title;
}


void Recipe::setTitle(const QString & theValue)
{
    if (d->title != theValue) {
        d->title = theValue;
    }
}


QString Recipe::author() const
{
    return d->author;
}


void Recipe::setAuthor(const QString& theValue)
{
    if (d->author != theValue) {
        d->author = theValue;
    }
}


QString Recipe::blurb() const
{
    return d->blurb;
}

void Recipe::setBlurb(const QString& theValue)
{
    if (d->blurb != theValue) {
        d->blurb = theValue;
    }
}

QString Recipe::effort() const
{
    return d->effort;
}

void Recipe::setEffort(const QString& theValue)
{
    if (d->effort != theValue) {
        d->effort = theValue;
    }
}

QString Recipe::genre() const
{
    return d->genre;
}

void Recipe::setGenre(const QString& theValue)
{
    if (d->genre != theValue) {
        d->genre = theValue;
    }
}

QString Recipe::preparationTime() const
{
    return d->preparationTime;
}

void Recipe::setPreparationTime(const QString& theValue)
{
    if (d->preparationTime != theValue) {
        d->preparationTime = theValue;
    }
}


QString Recipe::source() const
{
    return d->source;
}

void Recipe::setSource(const QString& theValue)
{
    if (d->source != theValue) {
        d->source = theValue;
    }
}


QString Recipe::yield() const
{
    return d->yield;
}

void Recipe::setYield(const QString& theValue)
{
    if (d->yield != theValue) {
        d->yield = theValue;
    }
}

int Recipe::numberOfIngredients() const
{
    return d->ingredients.size();
}


Ingredient Recipe::ingredient(int index) const
{
    if (!d->isIngredientIndexValid(index)) {
        return Ingredient();
    }

    return d->ingredients.at(index);
}

void Recipe::addIngredient(const Ingredient& theValue)
{
    d->ingredients.append(theValue);
}

void Recipe::addIngredient(double amount, const QString& unit, const QString& name)
{
    addIngredient(Ingredient(amount, unit, name));
}

void Recipe::setIngredient(int index, const Ingredient& theValue)
{
    if (!d->isIngredientIndexValid(index)) {
        return;
    }

    if (d->ingredients.at(index) != theValue) {
        d->ingredients.insert(index, theValue);
        d->ingredients.takeAt(index + 1);
    }
}

int Recipe::numberOfSteps() const
{
    return d->steps.size();
}

QString Recipe::step(int index) const
{
    if (!d->isStepIndexValid(index)) {
        return QString();
    }

    return d->steps.at(index);
}

void Recipe::addStep(const QString& theValue)
{
    d->steps.append(theValue);
}

void Recipe::setStep(int index, const QString& theValue)
{
    if (!d->isStepIndexValid(index)) {
        return;
    }

    if (d->steps.at(index) != theValue) {
        d->steps.insert(index, theValue);
        d->steps.takeAt(index + 1);
    }
}
