/****************************************************************************
 *  Copyright (C) 2009-2010 by Martin Engelmann, murphi.oss@googlemail.com  *
 *                                                                          *
 *  This program is free software; you can redistribute it and/or           *
 *  modify it under the terms of the GNU General Public License as          *
 *  published by the Free Software Foundation; either version 2 of          *
 *  the License or (at your option) any later version.                      *
 *                                                                          *
 *  This program is distributed in the hope that it will be useful,         *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>    *
 ****************************************************************************/

#include "recipeeditdialogpage_ingredients.h"

#include <kmessagebox.h>

#include "ui_RecipeEditDialogPage_ingredients.h"
#include "recipe.h"

RecipeEditDialogPageIngredients::RecipeEditDialogPageIngredients(QWidget* parent)
        : RecipeEditDialogPage(parent),
        ui(new Ui::RecipeEditDialogPage_ingredients)
{
    ui->setupUi(this);

    ui->pbAdd->setIcon(KIcon("list-add"));
    ui->pbRemove->setIcon(KIcon("list-remove"));
    ui->pbMoveUp->setIcon(KIcon("arrow-up"));
    ui->pbMoveDown->setIcon(KIcon("arrow-down"));
    ui->tblIngredients->setColumnCount(3);
    ui->tblIngredients->setHorizontalHeaderLabels(QStringList()
            << i18nc("@title:column", "Amount")
            << i18nc("@title:column", "Unit")
            << i18nc("@title:column", "Ingredient")
                                                 );

    connect(ui->pbAdd, SIGNAL(clicked()),
            this, SLOT(slotAdd()));
    connect(ui->pbRemove, SIGNAL(clicked()),
            this, SLOT(slotRemove()));
    connect(ui->pbMoveUp, SIGNAL(clicked()),
            this, SLOT(slotMoveUp()));
    connect(ui->pbMoveDown, SIGNAL(clicked()),
            this, SLOT(slotMoveDown()));
    connect(ui->tblIngredients, SIGNAL(itemSelectionChanged()),
            this, SLOT(slotEnableButtons()));
    connect(ui->tblIngredients, SIGNAL(itemChanged(QTableWidgetItem*)),
            this, SIGNAL(changed()));

    slotEnableButtons();
}


RecipeEditDialogPageIngredients::~RecipeEditDialogPageIngredients()
{
    delete ui;
}

void RecipeEditDialogPageIngredients::readFromRecipe(const Recipe& recipe)
{
    for (int i = 0; i < recipe.numberOfIngredients(); ++i) {
        addIngredient(recipe.ingredient(i));
    }
}

void RecipeEditDialogPageIngredients::writeToRecipe(Recipe& recipe) const
{
    for (int i = 0; i < ui->tblIngredients->rowCount(); ++i) {
        recipe.addIngredient(ingredient(i));
    }
}


void RecipeEditDialogPageIngredients::addIngredient(const Ingredient& ingredient)
{
    const int row = ui->tblIngredients->rowCount();
    ui->tblIngredients->setRowCount(row + 1);

    QTableWidgetItem* itemAmount = new QTableWidgetItem();
    itemAmount->setData(Qt::DisplayRole, ingredient.amount());
    itemAmount->setTextAlignment(Qt::AlignCenter);
    ui->tblIngredients->setItem(row, 0, itemAmount);

    QTableWidgetItem* itemUnit   = new QTableWidgetItem(ingredient.unit());
    ui->tblIngredients->setItem(row, 1, itemUnit);

    QTableWidgetItem* itemName   = new QTableWidgetItem(ingredient.name());
    ui->tblIngredients->setItem(row, 2, itemName);

    ui->tblIngredients->resizeColumnsToContents();
}

Ingredient RecipeEditDialogPageIngredients::ingredient(int index) const
{
    Ingredient result;
    result.setAmount(ui->tblIngredients->item(index, 0)->data(Qt::EditRole).value<double>());
    result.setUnit(ui->tblIngredients->item(index, 1)->data(Qt::EditRole).value<QString>());
    result.setName(ui->tblIngredients->item(index, 2)->data(Qt::EditRole).value<QString>());
    return result;
}

void RecipeEditDialogPageIngredients::slotAdd()
{
    const int row = ui->tblIngredients->rowCount();
    ui->tblIngredients->setRowCount(row + 1);

    ui->tblIngredients->setItem(row, 0, new QTableWidgetItem());
    ui->tblIngredients->item(row, 0)->setData(Qt::DisplayRole, 0.);
    ui->tblIngredients->item(row, 0)->setTextAlignment(Qt::AlignCenter);

    ui->tblIngredients->setItem(row, 1, new QTableWidgetItem());
    ui->tblIngredients->setItem(row, 2, new QTableWidgetItem());

    foreach(QTableWidgetItem* item, ui->tblIngredients->selectedItems()) {
        item->setSelected(false);
    }
    ui->tblIngredients->item(row, 0)->setSelected(true);
    ui->tblIngredients->scrollToItem(ui->tblIngredients->item(row,0));

    emit changed();
}

void RecipeEditDialogPageIngredients::slotRemove()
{
    const int rowToDelete = selectedRow(true);
    if (rowToDelete < 0) return;
    ui->tblIngredients->removeRow(rowToDelete);

    emit changed();
}

void RecipeEditDialogPageIngredients::slotMoveUp()
{
    const int rowToMove = selectedRow(true);
    swapRows(rowToMove, rowToMove - 1);
    ui->tblIngredients->selectRow(rowToMove - 1);

    emit changed();
}

void RecipeEditDialogPageIngredients::slotMoveDown()
{
    const int rowToMove = selectedRow(true);
    swapRows(rowToMove, rowToMove + 1);
    ui->tblIngredients->selectRow(rowToMove + 1);

    emit changed();
}

void RecipeEditDialogPageIngredients::slotEnableButtons()
{
    const int selected = selectedRow(false);
    const int rowCount = ui->tblIngredients->rowCount();

    bool remove = true;
    bool up = true;
    bool down = true;

    if (selected < 0 || selected >= rowCount) {
        remove = false;
        up = false;
        down = false;
    }

    if (selected == 0) {
        up = false;
    }

    if (selected == rowCount - 1) {
        down = false;
    }

    ui->pbRemove->setEnabled(remove);
    ui->pbMoveUp->setEnabled(up);
    ui->pbMoveDown->setEnabled(down);
}

int RecipeEditDialogPageIngredients::selectedRow(bool displayErrorMessage)
{
    QList<QTableWidgetItem*> selectedItems(ui->tblIngredients->selectedItems());

    if (selectedItems.size() == 0) {
        if (displayErrorMessage) {
            KMessageBox::sorry(this, i18nc("@info", "Please select an ingredient."));
        }
        return -1;
    }

    const int result = ui->tblIngredients->row(selectedItems.at(0));
    foreach(QTableWidgetItem* item, selectedItems) {
        if (ui->tblIngredients->row(item) != result) {
            if (displayErrorMessage) {
                KMessageBox::sorry(this,
                                   i18nc("@info", "You selected more than one ingredient.<nl/>"
                                         "Please select only one ingredient."));
            }
            return -1;
        }
    }

    return result;
}

void RecipeEditDialogPageIngredients::swapRows(int src, int dst)
{
    if (src < 0 || src >= ui->tblIngredients->rowCount()) return;
    if (dst < 0 || dst >= ui->tblIngredients->rowCount()) return;

    for (int column = 0; column < ui->tblIngredients->columnCount(); ++column) {
        QTableWidgetItem* oldSrc = ui->tblIngredients->takeItem(src, column);
        QTableWidgetItem* oldDst = ui->tblIngredients->takeItem(dst, column);
        ui->tblIngredients->setItem(src, column, oldDst);
        ui->tblIngredients->setItem(dst, column, oldSrc);
    }
}


#include "recipeeditdialogpage_ingredients.moc"
