/****************************************************************************
 *  Copyright (C) 2009-2010 by Martin Engelmann, murphi.oss@googlemail.com  *
 *                                                                          *
 *  This program is free software; you can redistribute it and/or           *
 *  modify it under the terms of the GNU General Public License as          *
 *  published by the Free Software Foundation; either version 2 of          *
 *  the License or (at your option) any later version.                      *
 *                                                                          *
 *  This program is distributed in the hope that it will be useful,         *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>    *
 ****************************************************************************/

#include "recipereaderwriterrecipexml.h"

#include <QtXml/QtXml>

#include <kurl.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <ksavefile.h>

#include "recipe.h"

RecipeReaderWriterRecipeXml::~RecipeReaderWriterRecipeXml()
{
}

bool RecipeReaderWriterRecipeXml::doReadRecipe(const KUrl& url, Recipe& recipe) const
{
    QFile file(url.toLocalFile());
    if (!file.open(QIODevice::ReadOnly)) {
        return false;
    }

    QDomDocument doc;
    QString errorStr;
    int errorLine;
    int errorColumn;
    if (!doc.setContent(&file, true, &errorStr, &errorLine, &errorColumn)) {
        KMessageBox::sorry(0,
                           i18nc("@info", "Error parsing <filename>%1</filename><nl/>"
                                 "at line <numid>%2</numid>, column <numid>%3</numid><nl/>"
                                 "<message>%4</message>",
                                 url.prettyUrl(), errorLine, errorColumn, errorStr));
        return false;
    }

    if (!doc.doctype().isNull()) {
        if (doc.doctype().name() != "recipe" ||
                doc.doctype().publicId() != "-//Happy-Monkey//DTD RecipeBook//EN") {
            KMessageBox::sorry(0, i18nc("@info", "The document type does not match!"));
            return false;
        }
    }

    QDomElement root = doc.documentElement();
    if (root.tagName() != "recipe") {
        return false;
    }

    recipe.setTitle(root.firstChildElement("title").text());
    if (recipe.title().isEmpty()) return false;

    QDomElement info = root.firstChildElement("recipeinfo");
    recipe.setAuthor(info.firstChildElement("author").text());
    recipe.setBlurb(info.firstChildElement("blurb").text());
    recipe.setEffort(info.firstChildElement("effort").text());
    recipe.setGenre(info.firstChildElement("genre").text());
    recipe.setPreparationTime(info.firstChildElement("preptime").text());
    recipe.setSource(info.firstChildElement("source").text());
    recipe.setYield(info.firstChildElement("yield").text());

    QDomElement ingredientlist = root.firstChildElement("ingredientlist");
    if (ingredientlist.isNull()) return false;
    QDomNodeList ingredients = ingredientlist.elementsByTagName("ingredient");
    for (int i = 0; i < ingredients.size(); ++i) {
        QDomElement ingredient = ingredients.item(i).toElement();
        if (ingredient.isNull()) continue;

        double amount = ingredient.firstChildElement("quantity").text().toDouble();
        QString unit = ingredient.firstChildElement("unit").text();
        QString name = ingredient.firstChildElement("fooditem").text();

        recipe.addIngredient(amount, unit, name);
    }

    QDomElement preparation = root.firstChildElement("preparation");
    if (preparation.isNull()) return false;
    QDomNodeList steps = preparation.elementsByTagName("step");
    for (int i = 0; i < steps.size(); ++i) {
        QDomElement step = steps.item(i).toElement();
        if (step.isNull()) continue;

        recipe.addStep(step.text());
    }

    return true;
}

bool RecipeReaderWriterRecipeXml::doWriteRecipe(const Recipe& recipe, const KUrl& url) const
{
    KSaveFile file(url.toLocalFile());
    if (!file.open()) return false;

    QXmlStreamWriter writer(&file);
    writer.setAutoFormatting(true);

    writer.writeStartDocument();
    writer.writeDTD("<!DOCTYPE recipe PUBLIC \"-//Happy-Monkey//DTD RecipeBook//EN\" \"http://www.happy-monkey.net/recipebook/recipebook.dtd\">");

    writer.writeStartElement("recipe");

    writer.writeTextElement("title", recipe.title());

    writer.writeStartElement("recipeinfo");
    writer.writeTextElement("author", recipe.author());
    writer.writeTextElement("blurb", recipe.blurb());
    writer.writeTextElement("effort", recipe.effort());
    writer.writeTextElement("genre", recipe.genre());
    writer.writeTextElement("preptime", recipe.preparationTime());
    writer.writeTextElement("source", recipe.source());
    writer.writeTextElement("yield", recipe.yield());
    writer.writeEndElement();

    writer.writeStartElement("ingredientlist");
    for (int i = 0; i < recipe.numberOfIngredients(); ++i) {
        writer.writeStartElement("ingredient");
        writer.writeTextElement("quantity", QString::number(recipe.ingredient(i).amount()));
        writer.writeTextElement("unit", recipe.ingredient(i).unit());
        writer.writeTextElement("fooditem", recipe.ingredient(i).name());
        writer.writeEndElement();
    }
    writer.writeEndElement();

    writer.writeStartElement("preparation");
    for (int i = 0; i < recipe.numberOfSteps(); ++i) {
        writer.writeTextElement("step", recipe.step(i));
    }
    writer.writeEndElement();

    writer.writeEndElement();

    writer.writeEndDocument();
    return file.finalize();
}
